@@ -1,6 +1,6 @@
 /*
  * activity.c: Define system activities available for sar/sadc.
- * (C) 1999-2021 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2022 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -53,12 +53,6 @@ struct act_bitmap cpu_bitmap = {
 	.b_size		= NR_CPUS
 };
 
-/* Interrupts bitmap */
-struct act_bitmap irq_bitmap = {
-	.b_array	= NULL,
-	.b_size		= NR_IRQS
-};
-
 
 /*
  * CPU statistics.
@@ -162,12 +156,12 @@ struct activity pcsw_act = {
 /* Interrupts statistics */
 struct activity irq_act = {
 	.id		= A_IRQ,
-	.options	= AO_COUNTED,
-	.magic		= ACTIVITY_MAGIC_BASE + 1,
+	.options	= AO_COUNTED + AO_MATRIX + AO_PERSISTENT,
+	.magic		= ACTIVITY_MAGIC_BASE + 2,
 	.group		= G_INT,
 #ifdef SOURCE_SADC
-	.f_count_index	= 1,	/* wrap_get_irq_nr() */
-	.f_count2_index	= -1,
+	.f_count_index	= 0,	/* wrap_get_cpu_nr() */
+	.f_count2_index	= 1,	/* wrap_get_irq_nr() */
 	.f_read		= wrap_read_stat_irq,
 #endif
 #ifdef SOURCE_SAR
@@ -175,7 +169,7 @@ struct activity irq_act = {
 	.f_print_avg	= print_irq_stats,
 #endif
 #if defined(SOURCE_SAR) || defined(SOURCE_SADF)
-	.hdr_line	= "INTR;intr/s",
+	.hdr_line	= "INTR;CPU*",
 #endif
 	.gtypes_nr	= {STATS_IRQ_ULL, STATS_IRQ_UL, STATS_IRQ_U},
 	.ftypes_nr	= {0, 0, 0},
@@ -186,7 +180,7 @@ struct activity irq_act = {
 	.f_svg_print	= NULL,
 	.f_raw_print	= raw_print_irq_stats,
 	.f_pcp_print	= pcp_print_irq_stats,
-	.f_count_new	= NULL,
+	.f_count_new	= count_new_int,
 	.item_list	= NULL,
 	.desc		= "Interrupts statistics",
 #endif
@@ -194,7 +188,7 @@ struct activity irq_act = {
 	.item_list_sz	= 0,
 	.g_nr		= 0,
 	.nr_ini		= -1,
-	.nr2		= 1,
+	.nr2		= -1,
 	.nr_max		= NR_IRQS + 1,
 	.nr		= {-1, -1, -1},
 	.nr_allocated	= 0,
@@ -202,7 +196,7 @@ struct activity irq_act = {
 	.msize		= STATS_IRQ_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= &irq_bitmap
+	.bitmap		= &cpu_bitmap
 };
 
 /* Swapping activity */
