@@ -1,6 +1,6 @@
 /*
  * pcp_def_metrics.c: Functions used by sadf to define PCP metrics
- * (C) 2019-2021 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 2019-2022 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -31,6 +31,44 @@
 #endif
 #endif /* HAVE_PCP */
 
+/*
+ ***************************************************************************
+ * Define PCP metrics for per-CPU interrupts statistics.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @cpu		CPU number (0 is cpu0, 1 is cpu1, etc.)
+ ***************************************************************************
+ */
+void pcp_def_percpu_int_metrics(struct activity *a, int cpu)
+{
+#ifdef HAVE_PCP
+	char buf[64];
+	struct sa_item *list = a->item_list;
+	static pmInDom indom = PM_INDOM_NULL;
+	static int inst = 0;
+
+	if (indom == PM_INDOM_NULL) {
+		/* Create domain */
+		indom = pmInDom_build(60, 40);
+
+		/* Create metric */
+		pmiAddMetric("kernel.percpu.interrupts",
+			     pmiID(60, 4, 1), PM_TYPE_U32, indom, PM_SEM_COUNTER,
+			     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
+	}
+
+	/* Create instance for each interrupt for the current CPU */
+	while (list != NULL) {
+		snprintf(buf, sizeof(buf), "%s::cpu%d", list->item_name, cpu);
+		buf[sizeof(buf) - 1] = '\0';
+
+		pmiAddInstance(indom, buf, inst++);
+		list = list->next;
+	}
+#endif /* HAVE_PCP */
+}
+
 /*
  ***************************************************************************
  * Define PCP metrics for CPU related statistics.
@@ -106,7 +144,7 @@ void pcp_def_cpu_metrics(struct activity *a)
 			}
 
 			else if (a->id == A_NET_SOFT) {
-				/* Create metrics for a_NET_SOFT */
+				/* Create metrics for A_NET_SOFT */
 				pmiAddMetric("network.softnet.processed",
 					     pmiID(60, 57, 0), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
 					     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
@@ -125,7 +163,7 @@ void pcp_def_cpu_metrics(struct activity *a)
 
 				pmiAddMetric("network.softnet.flow_limit_count",
 					     pmiID(60, 57, 5), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
-					     pmiUnits(0, -1, 1, 0, PM_TIME_SEC, PM_COUNT_ONE));
+					     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
 			}
 		}
 		else {
@@ -141,7 +179,12 @@ void pcp_def_cpu_metrics(struct activity *a)
 				pmiAddInstance(indom, buf, i - 1);
 			}
 
-			if (first) {
+			if (a->id == A_IRQ) {
+				/* Create per-CPU interrupts metrics */
+				pcp_def_percpu_int_metrics(a, i - 1);
+			}
+
+			else if (first) {
 				if (a->id == A_CPU) {
 					/* Create metrics for A_CPU */
 					pmiAddMetric("kernel.percpu.cpu.user",
@@ -214,6 +257,7 @@ void pcp_def_cpu_metrics(struct activity *a)
 						     pmiID(60, 57, 11), PM_TYPE_U64, indom, PM_SEM_COUNTER,
 						     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
 				}
+
 				first = FALSE;
 			}
 		}
@@ -250,32 +294,41 @@ void pcp_def_pcsw_metrics(void)
 void pcp_def_irq_metrics(struct activity *a)
 {
 #ifdef HAVE_PCP
-	int i, first = TRUE;
-	char buf[64];
-	pmInDom indom = pmiInDom(60, 4);
+	int first = TRUE, inst = 0;
+	struct sa_item *list = a->item_list;
+	pmInDom indom;
 
-	for (i = 0; (i < a->nr_ini) && (i < a->bitmap->b_size + 1); i++) {
+	if (!(a->bitmap->b_array[0] & 1))
+		/* CPU "all" not selected: Nothing to do here */
+		return;
 
-		/* Should current interrupt (including int "sum") be displayed? */
-		if (a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) {
+	/* Create domain */
+	indom = pmiInDom(60, 4);
 
-			if (!i) {
-				/* Interrupt "sum" */
-				pmiAddMetric("kernel.all.intr",
-					     pmiID(60, 0, 12), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
-					     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
-			}
-			else {
-				if (first) {
-					pmiAddMetric("kernel.all.interrupts.total",
-						     pmiID(60, 4, 0), PM_TYPE_U64, indom, PM_SEM_COUNTER,
-						     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
-					first = FALSE;
-				}
-				sprintf(buf, "%d", i - 1);
-				pmiAddInstance(indom, buf, i - 1);
-			}
+	/* Create instances and metrics for each interrupts for CPU "all" */
+	while (list != NULL) {
+		if (!strcmp(list->item_name, K_LOWERSUM)) {
+			/*
+			 * Create metric for interrupt "sum" for CPU "all".
+			 * Interrupt "sum" appears at most once in list.
+			 * No need to create an instance for it: It has a specific metric name.
+			 */
+			pmiAddMetric("kernel.all.intr",
+				     pmiID(60, 0, 12), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
+				     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
 		}
+		else {
+			if (first) {
+				/* Create metric for a common interrupt for CPU "all" if not already done */
+				pmiAddMetric("kernel.all.interrupts.total",
+					     pmiID(60, 4, 0), PM_TYPE_U64, indom, PM_SEM_COUNTER,
+					     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
+				first = FALSE;
+			}
+			/* Create instance */
+			pmiAddInstance(indom, list->item_name, inst++);
+		}
+		list = list->next;
 	}
 #endif /* HAVE_PCP */
 }
@@ -780,7 +833,7 @@ void pcp_def_net_nfs_metrics(void)
 	pmiAddInstance(indom, "read", 6);
 	pmiAddInstance(indom, "write", 8);
 	pmiAddInstance(indom, "access", 18);
-	pmiAddInstance(indom, "lookup", 4);
+	pmiAddInstance(indom, "getattr", 4);
 
 	pmiAddMetric("nfs.client.reqs",
 		     pmiID(60, 7, 4), PM_TYPE_U32, indom, PM_SEM_COUNTER,
@@ -830,7 +883,7 @@ void pcp_def_net_nfsd_metrics(void)
 	pmiAddInstance(indom, "read", 6);
 	pmiAddInstance(indom, "write", 8);
 	pmiAddInstance(indom, "access", 18);
-	pmiAddInstance(indom, "lookup", 4);
+	pmiAddInstance(indom, "getattr", 4);
 
 	pmiAddMetric("nfs.server.reqs",
 		     pmiID(60, 7, 12), PM_TYPE_U32, indom, PM_SEM_COUNTER,
