@@ -1,6 +1,6 @@
 /*
  * sar/sadc: Report system activity
- * (C) 1999-2021 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2022 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _SA_H
@@ -299,6 +299,7 @@
 #define NO_TM_START		0
 #define NO_TM_END		0
 #define NO_RESET		0
+#define NO_RANGE		0
 #define NON_FATAL		0
 #define FATAL			1
 #define C_SAR			0
@@ -634,11 +635,6 @@ struct file_header {
  * Base magical number for activities.
  */
 #define ACTIVITY_MAGIC_BASE	0x8a
-/*
- * Magical value used for activities with
- * unknown format (used for sadf -H only).
- */
-#define ACTIVITY_MAGIC_UNKNOWN	0x89
 
 /* List of activities saved in file */
 struct file_activity {
@@ -1002,6 +998,9 @@ struct activity {
 	 * separated with a '|' character.
 	 * For a given output, the first field corresponding to extended statistics
 	 * (eg. -r ALL) begins with a '&' character.
+	 * A field having '*' in its name means that all items will have its own column
+	 * (the first one will be displayed as "all", the following ones will get the '*'
+	 * character replaced with their item number (0, 1, etc.)
 	 */
 	char *hdr_line;
 	/*
@@ -1347,6 +1346,8 @@ struct sa_item {
 /*
  * Prototypes used to count new items
  */
+__nr_t count_new_int
+	(struct activity *, int);
 __nr_t count_new_net_dev
 	(struct activity *, int);
 __nr_t count_new_net_edev
@@ -1530,6 +1531,8 @@ char *get_fs_name_to_display
 	(struct activity *, uint64_t, struct stats_filesystem *);
 unsigned long long get_global_cpu_statistics
 	(struct activity *, int, int, uint64_t, unsigned char []);
+void get_global_int_statistics
+	(struct activity *, int, int, uint64_t, unsigned char []);
 void get_global_soft_statistics
 	(struct activity *, int, int, uint64_t, unsigned char []);
 void get_itv_value
@@ -1539,11 +1542,9 @@ void init_custom_color_palette
 int next_slice
 	(unsigned long long, unsigned long long, int, long);
 void parse_sa_devices
-	(char *, struct activity *, int, int *, int);
+	(char *, struct activity *, int, int *, int, int);
 int parse_sar_opt
 	(char * [], int *, struct activity * [], uint64_t *, int);
-int parse_sar_I_opt
-	(char * [], int *, uint64_t *, struct activity * []);
 int parse_sa_P_opt
 	(char * [], int *, uint64_t *, struct activity * []);
 int parse_sar_m_opt
