@@ -1,6 +1,6 @@
 /*
  * sadf_misc.c: Functions used by sadf to display special records
- * (C) 2011-2021 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 2011-2022 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -662,6 +662,7 @@ __printf_funct_t print_pcp_statistics(int *tab, int action, struct activity *act
 
 				case A_IRQ:
 					pcp_def_irq_metrics(act[p]);
+					pcp_def_cpu_metrics(act[p]);	/* For per_CPU int metrics */
 					break;
 
 				case A_SWAP:
@@ -1016,8 +1017,8 @@ __tm_funct_t print_raw_timestamp(void *parm, int action, char *cur_date,
 	static char pre[80];
 
 	if (action & F_BEGIN) {
-		snprintf(pre, 80, "%s%s", cur_time, strlen(cur_date) && utc ? " UTC" : "");
-		pre[79] = '\0';
+		snprintf(pre, sizeof(pre), "%s%s", cur_time, strlen(cur_date) && utc ? " UTC" : "");
+		pre[sizeof(pre) - 1] = '\0';
 		return pre;
 	}
 
@@ -1276,7 +1277,7 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 				printf("x%d", fal->nr2);
 			}
 			printf("\t(%d,%d,%d)", fal->types_nr[0], fal->types_nr[1], fal->types_nr[2]);
-			if ((p >= 0) && (act[p]->magic == ACTIVITY_MAGIC_UNKNOWN)) {
+			if ((p >= 0) && (act[p]->magic != fal->magic)) {
 				printf(_(" \t[Unknown format]"));
 			}
 			printf("\n");
@@ -1626,6 +1627,43 @@ __nr_t count_new_disk(struct activity *a, int curr)
 	return nr;
 }
 
+/*
+ ***************************************************************************
+ * Count the number of interrupts in current sample. Add each interrupt name
+ * to the linked list starting at @a->item_list.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ *
+ * RETURNS:
+ * Number of interrupts added to the list.
+ ***************************************************************************
+ */
+__nr_t count_new_int(struct activity *a, int curr)
+{
+	int i, nr = 0;
+	struct stats_irq *stc_cpuall_irq;
+
+	if (a->item_list)
+		/*
+		 * If a list already exists, do nothing. This means that a list has been
+		 * explicitly entered on the command line using option "--int=", or that
+		 * the list has already been created here (remember that the number of
+		 * interrupts cannot change in file: @nr2, the second matrix dimension,
+		 * is a constant).
+		 */
+		return 0;
+
+	for (i = 0; i < a->nr2; i++) {
+		stc_cpuall_irq = (struct stats_irq *) ((char *) a->buf[curr] + i * a->msize);
+
+		nr += add_list_item(&(a->item_list), stc_cpuall_irq->irq_name, MAX_SA_IRQ_LEN);
+	}
+
+	return nr;
+}
+
 /*
  ***************************************************************************
  * Init custom color palette used to draw graphs (sadf -g).
